
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MAINLAYER_HH_
#define _MAINLAYER_HH_

/**
 * @file MainLayer.hh
 * This is where all the fun begins.
 */

#include <iostream>
#include <string>

#include <boost/shared_ptr.hpp>

#include <Moof/Engine.hh>
#include <Moof/Layer.hh>
#include <Moof/Math.hh>


class MainLayer;
typedef boost::shared_ptr<MainLayer> MainLayerP;

struct MainLayer : public Mf::Layer
{
	MainLayer();
	~MainLayer();

	static MainLayerP alloc()
	{
		return MainLayerP(new MainLayer);
	}

	void pushed(Mf::Engine& engine);

	void draw(Mf::Scalar alpha) const;
	bool handleEvent(const Mf::Event& event);

	void quit();

private:

	/**
	 * Set OpenGL to a state we can know and depend on.
	 */
	void setupGL();
	void contextRecreated(const Mf::Notification* note);

	Mf::Engine* engine;
};


#endif // _MAINLAYER_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

