
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_ENGINE_HH_
#define _MOOF_ENGINE_HH_

#include <boost/shared_ptr.hpp>

#include <Moof/Exception.hh>
#include <Moof/Layer.hh>
#include <Moof/Math.hh>


namespace Mf {


// forward declarations
class Video;

struct Engine
{
	Engine(int argc, char* argv[], const std::string& name, 
			const std::string& iconFile, const std::string& configFile);
	~Engine() {}

	// get global instance
	static Engine& getInstance();

	void run();

	void setTimestep(Scalar ts);
	Scalar getTimestep() const;
	void setMaxFrameRate(long maxFps);
	long getMaxFrameRate() const;

	Video& getVideo() const;
	long getFrameRate() const;

	void push(LayerP layer);
	LayerP pop();
	// pops a specific layer and all layers above it
	LayerP pop(Layer* layer);
	void clear();

	struct Exception : public Mf::Exception
	{
		explicit Exception(unsigned error) :
			Mf::Exception(error) {}

		void raise()
		{
			throw *this;
		}
	};

private:
	class Impl;
	boost::shared_ptr<Impl> impl_;
};


} // namespace Mf

#endif // _MOOF_ENGINE_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

